package SCouT;

import java.io.IOException;
import java.util.Iterator;

import org.apache.hadoop.io.LongWritable;
import org.apache.hadoop.io.Text;
import org.apache.hadoop.mapred.JobConf;
import org.apache.hadoop.mapred.MapReduceBase;
import org.apache.hadoop.mapred.Mapper;
import org.apache.hadoop.mapred.OutputCollector;
import org.apache.hadoop.mapred.Reducer;
import org.apache.hadoop.mapred.Reporter;

public class Collapse {
	/**
	 * 
	 * Collapse CollapseMultipleModeMapper :     
	 * input:
	 * 	- I_1, I_2, ..., I_N, r:tensor_value
	 * output: (if current_mode == n)
	 *	- key(I_n), value(I_1, I_2, ..., I_N:r:tensor_value)
	 *
	 */
	public static class CollapseMultipleModeMapper extends MapReduceBase implements Mapper<LongWritable, Text, LongWritable, Text> {
		int current_mode = 0;	// current_mode
		int rank = 0; // rank
		public void configure(JobConf job) {
			current_mode = Integer.parseInt(job.get("current_mode"));
			rank = Integer.parseInt(job.get("rank"));
			System.out.println("CollapseMultipleModeMapper: current_mode = " + current_mode + "rank = " + rank);
		}

		public void map (LongWritable key, Text value, OutputCollector<LongWritable, Text> output, Reporter reporter) throws IOException {
			String line_text = value.toString(); // tensor: I_1, I_2, ..., I_N, r, tensor_value
			final String[] tokens = line_text.split("\t");
			LongWritable out_key = new LongWritable(Long.parseLong( tokens[current_mode-1]));
			Text out_value = new Text();
			String out_str = "m" + tokens[0];

			for(int i=1; i<tokens.length-2; i++) {
				out_str += "\t" + tokens[i];
			}
			out_str += ":" + tokens[tokens.length-2];
			out_str += ":" + tokens[tokens.length-1];

			out_value.set(out_str);

			output.collect(out_key, out_value); // key: product_mode_index, value: I_1, I_2, ..., I_N:r:tensor_value
		}
	}

	/**
	 * 
	 * Collapse CollapseMultipleModeMapper :     
	 * input:
	 * 	- row, col, rank, matrix_value
	 * output:
	 *	- key(row), value(row, col, rank, matrix_value)
	 *
	 */
	public static class CollapseMatrixMapper extends MapReduceBase implements Mapper<LongWritable, Text, LongWritable, Text> {
		int current_mode = 0;	// current_mode
		int rank = 0; // rank
		public void configure(JobConf job) {
			current_mode = Integer.parseInt(job.get("current_mode"));
			rank = Integer.parseInt(job.get("rank"));
			System.out.println("CollapseMultipleModeMapper: current_mode = " + current_mode + "rank = " + rank);
		}

		public void map (LongWritable key, Text value, OutputCollector<LongWritable, Text> output, Reporter reporter) throws IOException {
			String line_text = value.toString(); // tensor: I_1, I_2, ..., I_N, r, tensor_value
			final String[] tokens = line_text.split("\t");
			LongWritable out_key = new LongWritable(Long.parseLong( tokens[0]));
			Text out_value = new Text();
			String out_str = "m" + tokens[0];

			for(int i=1; i<tokens.length-2; i++) {
				out_str += "\t" + tokens[i];
			}
			out_str += ":" + tokens[tokens.length-2];
			out_str += ":" + tokens[tokens.length-1];

			out_value.set(out_str);
			output.collect(out_key, out_value); // key: product_mode_index, value: I_1, I_2, ..., I_N:r:tensor_value
		}
	}

	/**
	 * 
	 * CollapseTuckerReducer :
	 * input: (if current_mode == n)
	 *	- tensor: key(I_n), value(I_1, I_2, ..., I_N:r:tensor_value)
	 * 	- matrix: key(I_n), value(r, value)
	 * output: (if current_mode == n)
	 *	- (I_n, r_1, sum_val, r_2, sum_val,..., r_n, sum_val)
	 *
	 */
	public static class CollapseReducer extends MapReduceBase	implements Reducer<LongWritable, Text, LongWritable, Text> {
		int current_mode = 0;	// current_mode
		int rank = 0; // rank
		public void configure(JobConf job) {
			current_mode = Integer.parseInt(job.get("current_mode"));
			rank = Integer.parseInt(job.get("rank"));
			System.out.println("CollapseReducer: current_mode = " + current_mode + "rank = " + rank);
		}

		public void reduce (final LongWritable key, final Iterator<Text> values, final OutputCollector<LongWritable, Text> output, final Reporter reporter) throws IOException {
			double[] sum_val = new double[rank];

			for(int i=0; i<rank; i++) sum_val[i] = 0.0;

			while (values.hasNext()) {
				String line_text = values.next().toString();
				int idx = 0;
				Double cur_val = 0.0;
				if(line_text.charAt(0) == 'm') {
					final String[] tokens = line_text.split(":");
					idx = Integer.parseInt(tokens[1])-1;
					cur_val = Double.parseDouble(tokens[2]);
				} else {
					final String[] tokens = line_text.split("\t");
					idx = Integer.parseInt(tokens[0])-1;
					cur_val = Double.parseDouble(tokens[1]);
				}
				sum_val[idx] += cur_val;
			}

			String out_val_str = "1\t" + sum_val[0];
			for(int i=1; i<rank; i++) {
				out_val_str += "\t" + (i+1) + "\t" + sum_val[i];
			}
			output.collect(key, new Text(out_val_str));
		}
	}
}
