package SCouT;

import java.io.IOException;
import java.util.Iterator;

import org.apache.hadoop.io.LongWritable;
import org.apache.hadoop.io.Text;
import org.apache.hadoop.mapred.JobConf;
import org.apache.hadoop.mapred.MapReduceBase;
import org.apache.hadoop.mapred.Mapper;
import org.apache.hadoop.mapred.OutputCollector;
import org.apache.hadoop.mapred.Reducer;
import org.apache.hadoop.mapred.Reporter;

public class ConvertRowwise {
	/*********************************************************************************
	 * 
	 * ConvertRowwiseMapper: Converts factor matrix to rowwise formatted factor
	 * Input: 
	 *	- factor: key<>, value<mode, row, col, factor value>
	 * Output: 
	 *	- key<row>, value<col, factor value>  
	 *
	 *********************************************************************************/
	public static class ConvertRowwiseMapper extends MapReduceBase	implements Mapper<LongWritable, Text, LongWritable, Text>
	{
		int R=0;	// rank
		public void configure(JobConf job) {
			R = Integer.parseInt(job.get("R"));
			System.out.println("ConvertRowwiseMapper : R = " + R);
		}

		public void map (final LongWritable key, final Text value, final OutputCollector<LongWritable, Text> output, final Reporter reporter) throws IOException
		{
			String line_text = value.toString();
			final String[] line = line_text.split("\t");
			long row = Long.parseLong(line[1]);
			int col = Integer.parseInt(line[2]);
			output.collect( new LongWritable(row) , new Text("" + col +"\t"+ line[3]) );
		}
	}

	/*********************************************************************************
	 * 
	 * ConvertRowwiseReducer: Converts factor matrix to rowwise formatted factor
	 * Input: 
	 *	- key<row>, value<col, factor value>  
	 * Output: 
	 *	- key<mode>, value<row, col, factor value, col, factor value, ... , col, factor value>  
	 *
	 *********************************************************************************/
	public static class ConvertRowwiseReducer extends MapReduceBase implements Reducer<LongWritable, Text, LongWritable, Text>
	{
		int R=0;	// rank
		public void configure(JobConf job) {
			R = Integer.parseInt(job.get("R"));
			System.out.println("ConvertRowwiseReducer : R = " + R);
		}
		public void reduce (final LongWritable key, final Iterator<Text> values, final OutputCollector<LongWritable, Text> output, final Reporter reporter) throws IOException
		{
			double[] factor_val = new double[R];
			while (values.hasNext()) {
				String line_text = values.next().toString();
				final String[] tokens = line_text.split("\t");

				int col = Integer.parseInt(tokens[0]);
				double val = Double.parseDouble(tokens[1]);
				factor_val[col-1] = val;
			}
			String result = "";
			Text outvalue = new Text();
			for(int r=1; r<=R; r++) {
				if (r == 1) {
					result += r + "\t" + factor_val[r-1];
				} else {
					result += "\t" + r + "\t" + factor_val[r-1];
				}
			}
			outvalue.set(result);
			output.collect(key, outvalue ); // key: row, value: col value col value
		}
	}
}

