package SCouT;

import java.io.IOException;

import org.apache.hadoop.io.LongWritable;
import org.apache.hadoop.io.Text;
import org.apache.hadoop.mapred.JobConf;
import org.apache.hadoop.mapred.MapReduceBase;
import org.apache.hadoop.mapred.Mapper;
import org.apache.hadoop.mapred.OutputCollector;
import org.apache.hadoop.mapred.Reporter;

public class InitFactor {

	/**
	 * 
	 * InitFactorMapper: initialize factor matrix
	 * Input: 
	 * 	- row dimension, column dimension, factor indicator
	 * Output: 
	 *	- randomly generated factor matrix: key<row>, value<mode:column, factor value>
	 *
	 */
	public static class InitFactorMapper extends MapReduceBase implements Mapper<LongWritable, Text, LongWritable, Text> {
		long dim = 0;	// column dimension
		int nreducers = 0;	// number of reducers
		int current_mode = 0; // current_mode
		int rank = 0; // rank

		public void configure(JobConf job) {
			dim = Long.parseLong(job.get("dim"));
			current_mode = Integer.parseInt(job.get("current_mode"));
			rank = Integer.parseInt(job.get("rank"));
			nreducers = Integer.parseInt(job.get("nreducers"));

			System.out.println("InitFactorMapper : dim = " + dim + ", current_mode = " + current_mode + ",  rank = " + rank + ",  nreducer = " + nreducers);
		}
		public void map(LongWritable key, Text value, OutputCollector<LongWritable, Text> output, Reporter reporter) throws IOException {
			String line_text = value.toString();
			final String[] tokens = line_text.split("\t");

			for(int r=1; r<=rank; r++) {
				long row = 0;
				long i = 0;
				row = Integer.parseInt(tokens[0]) + (nreducers * i);
				while(row < dim) {
					output.collect(new LongWritable(current_mode), new Text("" + (row+1) + "\t" + r + "\t" + Math.random()));
					i++;
					row = Integer.parseInt(tokens[0]) + (nreducers * i);
				}
			}
		}
	}
}
