
package SCouT;
import java.io.IOException;
import java.util.Iterator;

import org.apache.hadoop.io.LongWritable;
import org.apache.hadoop.io.Text;
import org.apache.hadoop.mapred.JobConf;
import org.apache.hadoop.mapred.MapReduceBase;
import org.apache.hadoop.mapred.Mapper;
import org.apache.hadoop.mapred.OutputCollector;
import org.apache.hadoop.mapred.Reducer;
import org.apache.hadoop.mapred.Reporter;

public class MTMPrev {

	/**
	 * 
	 * MTMMapper: computes M^T M
	 * Input:
	 * 	- factor: key<>, value<row, col, factor value, col, factor value,  , col, factor value>
	 * Output:
	 *	- key<0>, value<String form of R-by-R matrix>
	 */
	public static class MTMPrevMapper extends MapReduceBase	implements Mapper<LongWritable, Text, LongWritable, Text>
	{
		int R = 0;	// number of columns

		public void configure(JobConf job) {
			R = Integer.parseInt(job.get("R"));

			System.out.println("MTMPrevMapper : R = " + R);
		}

		public void map (final LongWritable key, final Text value, final OutputCollector<LongWritable, Text> output, final Reporter reporter) throws IOException
		{
			String line_text = value.toString();
			final String[] tokens = line_text.split("\t");
			
			double[][] matrix = new double[R][R];
			int i,j;
			int cur_col_id;

			double[] vector = new double[R];
			for(i=0; i < R; i++)
				vector[i] = 0;

			for( i = 1; i <= tokens.length - 1; i += 2 )  {
				cur_col_id = Integer.parseInt( tokens[i] );

				double cur_val = Double.parseDouble( tokens[i+1] );

				vector[ cur_col_id -1] = cur_val;
			}

			for(i =0; i < R; i++) {
				for(j=0; j< R; j++) {
					matrix[i][j] = vector[i] * vector[j];
				}
			}

			String out_str = RRmatrixToStr( matrix, R );
			
			output.collect(new LongWritable(0), new Text(out_str) );
		}
	}

	/**
	 * 
	 * MTMPrevReducer: computes M^T M
	 * Input:
	 * 	- key<0>, value<String form of R-by-R matrix>
	 * Output:
	 *	- key<0>, value<String form of R-by-R matrix>
	 */
	public static class MTMPrevReducer extends MapReduceBase implements Reducer<LongWritable, Text, LongWritable, Text>
	{
		int R = 0; // number of columns

		public void configure(JobConf job) {
			R = Integer.parseInt(job.get("R"));

			System.out.println("MTMPrevReducer : R = " + R);
		}

		public void reduce (final LongWritable key, final Iterator<Text> values, final OutputCollector<LongWritable, Text> output, final Reporter reporter) throws IOException
		{
			double[][] sum_matrix = new double[R][R];
			int i,j;

			for(i =0; i < R; i++) {
				for(j=0; j< R; j++) {
					sum_matrix[i][j] = 0;
				}
			}

			while (values.hasNext()) {
				String line_text = values.next().toString();
				String[] tokens = line_text.split("\t");
				//System.out.println("[Debug MTMReducer] input: " + line_text);
				double[][] cur_matrix = StringToRRmatrix( tokens, R );

				for(i =0; i < R; i++) {
					for(j=0; j< R; j++) {
						sum_matrix[i][j] += cur_matrix[i][j];
					}
				}
			}

			String out_str = RRmatrixToStr( sum_matrix, R );
			//System.out.println("[Debug MTMMapper] key: 0 out_value: " + out_str);
			output.collect(new LongWritable(0), new Text(out_str) );
		}
	}

	// Converting R by R matrix to string
	// Input: 
	//	- matrix: input matrix to convert
	//	- R: dimension of matrix
	// Output:
	//	- string of result matrix
	public static String RRmatrixToStr( double[][] matrix, int R ) {
		String result_str = "";
		int i, j;

		for(i =0; i < R; i++) {
			for(j=0; j< R; j++) {
				if( result_str.length() > 0 )
					result_str += "\t";
				result_str += matrix[i][j];
			}
		}

		return result_str;
	}

	// Converting string to R by R matrix
	// Input: 
	//	- tokens: input string to convert
	//	- R: dimension of matrix
	// Output:
	//	- double array of result matrix
	public static double[][] StringToRRmatrix( String[] tokens, int R) {
		double[][] matrix = new double[R][R];
		int i,j;

		int cur = 0;
		for(i = 0; i < R; i++) {
			for(j=0; j < R; j++) {
				matrix[i][j] = Double.parseDouble( tokens[cur++] ) ;
			}
		}

		return matrix;
	}
}
