package SCouT;

import java.io.IOException;

import org.apache.hadoop.io.IntWritable;
import org.apache.hadoop.io.LongWritable;
import org.apache.hadoop.io.Text;
import org.apache.hadoop.mapred.JobConf;
import org.apache.hadoop.mapred.MapReduceBase;
import org.apache.hadoop.mapred.Mapper;
import org.apache.hadoop.mapred.OutputCollector;
import org.apache.hadoop.mapred.Reporter;

public class MatrixMatrixStrProduct {
	/**
	 * MapOnlyMatrixMatrixStr: perform multiplication of matrix and string form of matrix
	 * Input: 
	 *	- factor: key<>, value<row, col, factor value, col, factor value, ... , col, factor value>
	 * Output: 
	 * 	- key<row>, value<col, result value, col, result value, ..., col, result value>
	 */
	public static class MatrixMatrixStrProductMapper extends MapReduceBase	implements Mapper<LongWritable, Text, IntWritable, Text>
	{
		String matrix_str= ""; // string form of matrix
		double [][] matrix_val = null; // matrix to store input
		int rowdim = 0;	// dimension of row
		int coldim = 0; // dimension of column
		int current_mode = 0; // current mode

		public void configure(JobConf job) {
			matrix_str = job.get("matrix_str");
			rowdim = Integer.parseInt(job.get("rowdim"));
			coldim = Integer.parseInt(job.get("coldim"));
			current_mode = Integer.parseInt(job.get("current_mode"));
			matrix_val = new double[rowdim][coldim];

			String[] tokens = matrix_str.split("\t");
			for(int i = 0; i < rowdim; i++) {
				for(int j=0; j < coldim; j++) {
					matrix_val[i][j] = Double.parseDouble( tokens[i*coldim +j] );
					System.out.println("matrix_val[" + i + "][" + j + "]=" + matrix_val[i][j] );
				}
			}

			System.out.println("MatrixMatrixStrProductMapper : current_mode = " + current_mode + " rowdim = " + rowdim + " coldim = " + coldim + " matrix_str = [" + matrix_str + "]");
		}

		public void map (final LongWritable key, final Text value, final OutputCollector<IntWritable, Text> output, final Reporter reporter) throws IOException
		{
			String line_text = value.toString();
			System.out.println("[Debug MatrixMatrixStrProductMapper] input: "+line_text);
			final String[] tokens = line_text.split("\t");

			int row = Integer.parseInt(tokens[0]);
			int cur_col_id;

			double[] result_val = new double[coldim];
			for(int i=0; i < coldim; i++) {
				result_val[i] = 0;
			}

			for(int i = 1; i < tokens.length; i+=2)  {
				cur_col_id = Integer.parseInt( tokens[i] );

				double cur_val = Double.parseDouble( tokens[i+1] );

				for(int j=0; j<coldim; j++) {
					result_val[j] += cur_val * matrix_val[cur_col_id-1][j];
				}
			}

			IntWritable outkey = new IntWritable();
			Text outvalue = new Text();
			outkey.set(current_mode);
			for(int i = 0; i < coldim; i++) {
				outvalue.set("" + row + "\t" + (i+1) + "\t" + result_val[i]);
				output.collect(outkey, outvalue);
			}
		}
	}
}
